-- contains just longer or multiple times used strings

local addonName, AT = ...

local Constants = AT.Constants

local customEscapeSequences = {
  ['|cy1'] = '|c'..Constants.COLOR_YELLOW_HEX,
  ['|cy2'] = '|cfffff17c',
  ['|cgr'] = '|cffc1c1c1',
  ['|cre'] = '|cffff3535',
  ['|cbl1'] = '|cff427df4',
  ['|cbl2'] = '|cff00ddff',
  ['|cptype'] = '|c'..Constants.COLOR_PROFILE_TYPE_HEX,
  ['|cpname'] = '|c'..Constants.COLOR_PROFILE_NAME_HEX,
}

local Strings = {}
AT.Strings = Strings

Strings.welcomeMsg = string.format([[Welcome to %s!
Visit the settings menu to customize this addon to your needs. You can for example customize the appearance of frames, enable / disable features, setup filters or create profiles which can be changed via macros for different situations.
Type in |cffffd43a/SpellstealMaster|r or open the "AddOns" tab in the interface settings to access the settings menu.
]], Constants.ADDON_DISPLAY_NAME)

Strings.otherSettingsHint = [[
Select one of categories on the left side for more settings.
]]

Strings.bugReportHint = [[
|cy1Report bugs to:|r
https://wow.curseforge.com/projects/spellsteal-master/issues
]]

Strings.testFramesEnabled = [[
Test frames|cff09e226 enabled|r. They will disappear automatically as soon as your characters starts moving.
|cgrNote: These frames are primarily intended to customize the appearance and therefore do not respond to all settings.|r
]]

Strings.testFramesDisabled = "Test frames|cffffff00 disabled|r."

Strings.descriptionExperimentalSettings = "Contains features that are not yet 100% working, but could be still useful for some players."

Strings.tooltipAnchor = "Sets the anchor point relative to the screen."

Strings.tooltipPivot = [[Sets the zero point of the frame.
If for example the value is set to "Top Left", the frame will grow to the right and bottom.]]

Strings.profileCommands = {
  create = "/SpellstealMaster Profiles Create [Type] [Name]",
  load = "/SpellstealMaster Profiles Load [Type] [Name]",
  loadnext = "/SpellstealMaster Profiles LoadNext [Type]",
  delete = "/SpellstealMaster Profiles Delete [Type] [Name]",
  copy = "/SpellstealMaster Profiles Copy [Type] [Name] [NewName]",
  rename = "/SpellstealMaster Profiles Rename [Type] [Name] [NewName]",
  list = "/SpellstealMaster Profiles List\nor\n/SpellstealMaster Profiles List [Type]",
  help = "/SpellstealMaster Profiles Help",
}

Strings.descriptionProfiles = [[
Profiles are not yet available in the settings menu, but you can use its full functionality via the following slash commands:

|cy2/SpellstealMaster |cff51efcfProfiles |cbl2Create |cptype[Type] |cpname[Name]
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2Load |cptype[Type] |cpname[Name]
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2LoadNext |cptype[Type]
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2Delete |cptype[Type] |cpname[Name]
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2Copy |cptype[Type] |cpname[Name] [NewName]
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2Rename |cptype[Type] |cpname[Name] [NewName]
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2List
|cy2/SpellstealMaster |cff51efcfProfiles |cbl2List |cptype[Type]

|cptype[Type]|r = |cptypeGeneral|r or |cptypeFilter|r
|cpname[Name]|r = any profile name

|cy1Example:|r
/spellstealmaster profiles create filter pvp

|cy1Notes:|r
- Upper and lower case will be ignored.
- The keyword |cff51efcfProfiles|r can be replaced with |cff51efcfProfile|r or |cff51efcfp|r.

As you have already seen, profiles are divided into two types:
- |cptypeGeneral|r: contains all basic and appearance settings
- |cptypeFilter|r: contains all settings that are used to filter buffs

|cy1Hints:|r
- You can change profiles via makros. Escpecially the |cbl2LoadNext|r command can be useful for this.]]

Strings.profileCmdFailed = [[
Invalid command usage. Excpected usage:
|cy2%s|r
Type in |cy2/SpellstealMaster Profiles Help|r to get more information.
]]

Strings.profileCmdInvalid = [[
Invalid profile command.
Type in |cy2/SpellstealMaster Profiles Help|r to get a list of all available profile commands.
]]

Strings.testSpots = [[
|cy1Good test spots to setup this addon with real data:|r


|cy2Stratholme (Service Entrance):|r
Entrance: Eastern Plaguelands
/way 43.45, 19.40

|cy2Elwynn Forest:|r
/way 29.5, 58.25

|cy2Isle of Quel'Danas|r
/way 44.85, 38.75
]]

Strings.descriptionCsv = [[
|cy1BASIC HOW TO:|r
Just write all the spell IDs and names you want to filter out into the text field, separated by commas. Spaces and line breaks between spells will be ignored.

|cy1ADVANCED TIPS:|r
- You can create comments by writing text in parentheses.
- If you want to use reserved characters in spell names, you have to write a backslash before them. The reserved characters are |cre,|r and |cre(|r and |cre)|r.
]]

Strings.descriptionFilterSoft = [[
Spells that are filtered out with this filter will be completely hidden in the overview frame and the mouseover tooltip, as well as on nameplates.
In the target and focus frame they will only appear if there is at least one other spell that is not filtered out.
]]

Strings.filterContextualLua1 = string.format([[
%s

|cy1You can find a description how to write a Lua filter here:|r
]], Strings.descriptionFilterSoft)

Strings.filterContextualLua2 = [[
|cy2Do not paste here code from persons you do not trust.|r
]]

Strings.filterContextualCsv = string.format([[
%s

%s
]], Strings.descriptionFilterSoft, Strings.descriptionCsv)

Strings.filterGlobalCsv = string.format([[
Unlike the “soft" filter, all spells that are filtered by the “hard" filter are completely ignored in every scenario. This type of filtering can be for example used to filter out false positives (btw, if filtering false positives is your use case, consider reporting them as bug so that they can be added to the internal blacklist of this addon).

%s
]], Strings.descriptionCsv)

Strings.howToFilterContextualLua = [[
|cy2Warning: Basic Lua skills are required|r


Roughly summarized, you can write a function which is used for additional buff filtering. Two tables with information about the buff and its owning unit are passed to the Lua function and a Boolean indicating whether to show this buff or not is expected as return value.

In order to determine which value you want to return, you can access the game API:
https://wow.gamepedia.com/World_of_Warcraft_API
https://wowwiki.fandom.com/wiki/World_of_Warcraft_API


|cy1TABLES PASSED TO THE FUNCTION:|r

unitInfo:
  id   |cgr-   String: id of the unit, e.g. ‘target’, ‘nameplate5’, ‘boss1’|r
  guid   |cgr-   String: GUID of the unit|r

buffInfo:
  name   |cgr-   String: buff name|r
  count   |cgr-   Number: number of times this buff has been applied, 0 if buff is not stackable|r
  buffType   |cgr-   String: buff type, e.g. ‘Magic’|r
  expirationTime   |cgr-   Number: remaining buff time in seconds (rounded)|r
  expirationTimeRaw   |cgr-   Number: system uptime in seconds when the buff will expire|r
  unitCaster   |cgr-   String: unit id that applied the buff, e.g. ‘nameplate1’|r
  spellId   |cgr-   Number: buff id|r
  canApplyAura   |cgr-   Boolean: true if player can apply this buff|r
  isCastByPlayer   |cgr-   Boolean: true if buff applied by a player|r
  isBossBuff   |cgr-   Boolean: true if buff applied by a boss|r
  nameplateShowAll   |cgr-   Boolean: true if the buff should be shown on nameplates. This refers to functionality of the WoW client and not to the nameplate features of this addon.|r

Source for most of the descriptions:
https://wow.gamepedia.com/API_UnitBuff


|cy1EXPECTED RETURN VALUE:|r

Boolean: true if the buff should be shown, false otherwise.


|cy1WHEN EXACTLY THIS FUNCTION GETS CALLED?|r

The Lua function gets called for each buff that is:
- flagged as steal or purge-able by the game
- not blacklisted by the addon‘s internal blacklist for false positives
- not filtered out by the “soft“ or “hard” filter created by the player
- is not a hardcoded group buff (if “ignore group buffs” is set to true)
After the Lua function got called, other settings like the minimum expiration time gets applied. So it can happen that a buff will be filtered out later, even though it has been declared as valid by the Lua function.


|cy1EXAMPLE 1:|r Show only buffs on player characters

return UnitIsPlayer(unitInfo.id)


|cy1EXAMPLE 2:|r Show only buffs that where applied from a specific class

-- can be nil when no unit id exists for the unit
if buffInfo.unitCaster == nil then
  return false
end
localizedClass, englishClass = UnitClass(unitInfo.id)
return englishClass == ‘DRUID’
]]

local function postProcessStrings(aTable)
  for k, v in pairs(aTable) do
    local t = type(v)
    if t == 'string' then
      local str = v
      for k, v in pairs(customEscapeSequences) do
        str = gsub(str, k, v)
      end
      aTable[k] = strtrim(str)
    elseif t == 'table' then
      postProcessStrings(v)
    end
  end
end
postProcessStrings(Strings)
